//======================================================================
//-----------------------------------------------------------------------
/**
 * @file		FndSHA256.h
 * @brief		SHA256 NX t@C
 *
 * @author		t.sirayanagi
 * @version		1.0
 *
 * @par			copyright
 * Copyright (C) 2010-2011 Takazumi Shirayanagi\n
 * The new BSD License is applied to this software.
 * see iris_LICENSE.txt
*/
//-----------------------------------------------------------------------
//======================================================================
#ifndef INCG_IRIS_FndSHA256_H_
#define INCG_IRIS_FndSHA256_H_

//======================================================================
// include
#include "FndHash.h"
#include "../../math/hash/MathSHA256.h"

namespace iris {
namespace fnd
{

//======================================================================
// class
/// SHA256 nbVNX
class CSHA256 : public IHash
{
public:
	static const int HASHSIZE	= math::SHA256_HASHSIZE;
	typedef math::SHA256CONTEXT	CONTEXT;
protected:
	CONTEXT		m_Context;	//!< ReLXg

public:
	// RXgN^
	CSHA256(void);
	CSHA256(const u8* lpBuffer, size_t uLength)	{ Encode(lpBuffer, uLength); }
	template<typename TN>
	CSHA256(const TN* lpBuffer, size_t uLength)	{ Encode(lpBuffer, uLength); }
	template<typename TN, size_t SIZE>
	CSHA256(const TN (&lpBuffer)[SIZE])			{ Encode<TN, SIZE>(lpBuffer); }
	// fXgN^
	virtual ~CSHA256(void);
public:

	// 
	virtual void	Init(void);
	// NA
	virtual void	Clear(void);
	// XV
	virtual void	Update(const u8* lpBuffer, size_t uLength);
	// ŏIXV
	virtual void	Final(void);
	// nbV̏o
	virtual void	GetDigest(u8* lpBuffer)	const;
	/// o
	virtual void	ToStringA(LPSTR  lpString, size_t uSize, u8* lpBuffer)	const;
	virtual void	ToStringW(LPWSTR lpString, size_t uSize, u8* lpBuffer)	const;
	/// o
	virtual void	ToStringA(LPSTR  lpString, size_t uSize)	const;
	virtual void	ToStringW(LPWSTR lpString, size_t uSize)	const;
	// nbVTCY̎擾
	virtual u32		GetHashSize(void)	const	{ return HASHSIZE; }

public:
	// ReLXg̎擾
	const CONTEXT&	GetContext(void)	const	{ return m_Context; }
};

}	// end of namespace fnd
}	// end of namespace iris

#endif
